// **********************************************************************
// SCIDYNE   649 School Street   Pembroke, MA 02359-3649 USA
// Tel: (781) 293-3059   Fax: (781) 293-4034   URL: www.scidyne.com
// **********************************************************************
// Project:  DIO96-104 Demonstration Program
// File:     D96_EX1.CPP
// Revision: 1.00
// Date:     02-07-03
// Author:   Mark W. Durgin
// *************************************************************************
// This C program demonstrates the basic operation of the SCIDYNE DIO96-104
// digital Input/Output module (Model 100-7618). For clarity and ease in
// understanding the program has been written with simplicity taking
// precedence over robustness. It is supplied without any implied or
// expressed warranties or claims. The user accepts all consequences
// resulting from its use and has the sole responsibility in determining its
// suitability for each application.
//
// The DIO96-104 uses four industry standard 82C55 programmable Peripheral
// Interface chips. The 82C55 is very versatile and can be configured in
// many different ways. It is beyond the scope of this example to show each
// operational mode. The user is advised to refer to the 82C55 data sheet
// for complete programming information. This data sheet is available at the
// SCIDYNE web site www.scidyne.com as well as other sources.

// The program controls the state of GROUP0:PORTA:BIT7 by means of
// user entered keystrokes. This example program only controls the
// first 82C55 chip. However, the other three chips are easily added.
// After setting up the user display the program initializes the control
// register of the first 82C55 chip (GROUP0).  The MODE in this example
// sets PORTA as outputs and PORTB and PORTC as inputs. The keyboard is
// then examined in a continuous loop. Pressing the "0" key will cause the
// program to write a value to GROUP0_PORTA which resets bit#7. Pressing
// the "1" key will cause the program to write a value to GROUP0_PORTA
// which sets bit#7. Pressing the "X" key ends the program. In addition,
// PORTB is read and the state of its bits are displayed. Because the
// DIO96-104 does not use pull-up or pull-down resistors, the state of the
// PORTB bits will be random until hard logic levels are applied externally.
//
// *************************************************************************
// History:
//   02-06-03  R1.00  Original Code
// *************************************************************************

// --- Program includes
#include <stdio.h>
#include <dos.h>
#include <conio.h>

// --- Define program constants
#define	TRUE								1									// Logic TRUE state
#define FALSE								!TRUE							// Logic FALSE state
#define	BASE_ADDR 					0x300							// Define where in IO map DIO96 resides

#define GROUP0_PORTA				BASE_ADDR+0       // Group#0 82C55 PORT-A
#define GROUP0_PORTB				BASE_ADDR+1       // Group#0 82C55 PORT-B
#define GROUP0_PORTC				BASE_ADDR+2       // Group#0 82C55 PORT-C
#define GROUP0_DIO_CTRL			BASE_ADDR+3       // Group#0 82C55 Control Register
#define GROUP0_DIO_MODE			0x8B 		          // Defines MODE of Group #0 82C55

// ********************************************************************
// ** This is the main loop
// ********************************************************************
void main(void)
{
	//--- Declare local variables
	unsigned char       byte;				// Used to hold image of PORTB
  int                 bit;        // Used as loop index

	//--- Initz the program
	//--- Clear the display and show a banner
	clrscr();
	printf ("------------------------------\n");
	printf ("         SCIDYNE\n");
	printf ("DIO96-104 Demonstration Program\n");
	printf ("    d96_ex1.cpp     V1.00\n");
	printf ("------------------------------\n\n");
	printf ("Measure logic level for Group#0, PORTA, Bit#7\n");
	printf ("on connector J3 Pin #1\n");
	printf ("Press '0' to set to LOGIC 0\n");
	printf ("Press '1' to set to LOGIC 1\n");
	printf ("Press 'X' to eXit program\n\n");
	printf ("GROUP#0 PORTB BITS:\n");
	printf ("0 1 2 3-4 5 6 7");

	//--- Setup and initz Group #0 digital IO
	outp(GROUP0_DIO_CTRL,GROUP0_DIO_MODE);   	// Make all ports of 82C55 outputs
	outp(GROUP0_PORTA,0);  				            // Force all bits of GROUP0_PORTA to 0 (low)
	outp(GROUP0_PORTB,0);  				            // Force all bits of GROUP0_PORTB to 0 (low)
	outp(GROUP0_PORTC,0);  				            // Force all bits of GROUP0_PORTC to 0 (low)

  //--- This is the main loop
  while(TRUE)
	{
		//--- Check if user is making changes to bit#7 output or
		//    requesting the program to end
		if(kbhit())
		{
			switch (getch())  // Read the keystroke
			{
				case '0':  // Set Bit #7 "0"
				{
          outp(GROUP0_PORTA, 0x0);   // Make bit #7 LOGIC "0"
          break;
				}

				case '1': // Set Bit #7 "1"
				{
          outp(GROUP0_PORTA, 0x80);   // Make bit #7 LOGIC "1"
          break;
				}

				case 'x':
				case 'X':  // Quit the program
				{
					return;
					break;
				}

				default:
					break;
			}
		}

		//--- Read PORTB and display the bits indiviually
    byte = inp(GROUP0_PORTB);   // Read PORTB
    gotoxy (1,15); // Locate the display cursor

    for (bit = 0; bit < 8; bit++) // test each bit
    {
      if (byte & (1 << bit))  // Test if bit is 1 or 0
      {
        printf("1 ");  // Show bit as 1
      }
      else
      {
        printf("0 ");  // Show bit as 0
      }
    }
	}
}



